<template>
    <div>
        <el-row type="flex" class="rowStyle" :class="{'new': !agentInfo.customerViewTime }">
            <el-col class="flex_column pl-5" :xs="5" :sm="3">
                <UiAvatar
                    :size="50"
                    :agentNo="agentInfo.agentNo"
                    @click.native="showAgentDetail(agentInfo.agentNo)"
                ></UiAvatar>
                <div v-if="!hideReviews" style="display: flex; justify-content: flex-start; align-items: center; margin-top: 6px; font-size: 12px">
                    <i v-if="isAppointment ? agentInfo.appointmentScore : agentInfo.avgScore"
                      class="icon-star pam-icon icon--yellow satisfaction"
                       style="margin-top: 0"></i>
                    <template v-if="isAppointment">
                      <template v-if="agentInfo.appointmentStatus === 'close'
                                    || agentInfo.appointmentStatus === 'done'">
                        <span v-if="agentInfo.appointmentScore">
                            {{ agentInfo.appointmentScore }}
                        </span>
                        <div class="unfilled text--center "
                            style="display:flex"
                            v-else>未填<br />滿意度</div>
                      </template>
                    </template>
                    <template v-else>
                      <span v-if="agentInfo.avgScore">{{ agentInfo.avgScore }}</span>
                      <span class="unfilled text--center "
                          style="display:flex" v-else>尚無<br />滿意度</span>
                    </template>

                </div>
            </el-col>
            <el-col :xs="10" :sm="15">
                <div class="smTxt_bold name">{{agentInfo.name}}</div>
                <div class="professionals">
                    <span
                        class="professionalsTxt"
                        v-for="(expertise, index) in agentInfo.expertise"
                        :key="index"
                    >#{{expertise}}</span>
                </div>
                <div
                    class="delete"
                    v-if="showRemoveBtn"
                    @click="isRemoveAgentPopup = true"
                >移除</div>
                <div
                    v-if="notScoreAppointmentYet && (agentInfo.appointmentStatus === 'closed' || agentInfo.appointmentStatus === 'done')"
                    class="text--primary text--underline cursor--pointer xsTxt text--bold"
                    @click="reviewsBtn = true">給予滿意度評分</div>
            </el-col>
            <el-col class="flex_column" :xs="9" :sm="6">
                <el-button
                    class="smTxt_bold outline_btn"
                    @click="reservedOrShowAppointmentInfo"
                    :class="actionBtnStyle"
                >{{ actionBtnLabel }}</el-button>
                <div class="updateTime mt-10">
                    {{ (isAppointment ? agentInfo.appointmentLastModifiedDate : agentInfo.updateTime)  | formatDate }}
                </div>
            </el-col>
        </el-row>

        <Ui-Dialog
            :isVisible.sync="isVisibleDialog"
            :width="width"
            class="pam-myDemand-dialog pam-dialog-reserved"
        >
            <div v-if="appointmentDetail">
                <h5 class="subTitle text--center mb-30">預約成功</h5>
                <p class="smTxt">{{appointmentDetail.appointmentDate | formatDate}} 建立</p>
                <div class="reserved-info">
                    <p>姓名：{{appointmentDetail.name}}</p>
                    <p>電話：{{appointmentDetail.phone}}</p>
                    <p>Email：{{appointmentDetail.email}}</p>
                    <p>性別：{{gender}}</p>
                    <p>年齡：{{appointmentDetail.age | toAgeLabel }}</p>
                    <p>職業：{{appointmentDetail.job}}</p>
                    <p>需求：{{appointmentDetail.requirement.split(',').join('、')}}</p>
                    <p
                        v-for="(item, index) in hopeContactTime"
                        :key="index"
                    >連絡時段{{index + 1 | formatNumber}}：{{ item | formatHopeContactTime }}</p>
                    <div v-if="appointmentDetail.appointmentScore">
                        <div class="mdTxt mt-10 mb-10">滿意度</div>
                        <el-rate
                        :value="appointmentDetail.appointmentScore"
                        class="pam-myDemand-dialog__rate"
                        disabled>
                        </el-rate>
                    </div>
                </div>

                <div v-if="notScoreAppointmentYet && (agentInfo.appointmentStatus === 'closed' || agentInfo.appointmentStatus === 'done')" class="reserved-btn">
                    <el-button type="primary"
                        @click.native="reviewsBtn = true">給予滿意度評分</el-button>
                </div>

                <div v-if="agentInfo.contactStatus === 'reserved' && !isAppointment" class="text--center mt-10">
                    <el-button @click="isCancelPopup = true">取消預約</el-button>
                    <el-button @click="edit" type="primary">編輯</el-button>
                </div>
            </div>
        </Ui-Dialog>

        <PopUpFrame :isOpen.sync="reviewsBtn" class="reviewDialog-content">
            <div class="mdTxt pam-dialog-review">
                保險顧問滿意度
                <span class="hint">選取星星</span>
                <div class="mt-30 review-content">
                    <UiAvatar :size="80" :agentNo="agentInfo.agentNo"></UiAvatar>
                    <div class="review-text">對於顧問
                        <span class="text--primary">{{agentInfo.name}}</span>
                        的整體服務，您給予幾顆星評價？
                    </div>
                </div>

                <div class="review-score">
                    <el-rate v-model="inputScore" class="pam-rate mt-30"></el-rate>
                </div>

                <div class="review-btn">
                    <el-button
                        type="primary"
                        :disabled="!inputScore"
                        @click="userReviewsConsultants">送出</el-button>
                </div>
            </div>
        </PopUpFrame>

        <PopUpFrame :isOpen.sync="isCancelPopup">
            <div class="text--center mdTxt">是否取消此筆預約？</div>
            <div class="text--center mt-30">
                <el-button @click="isCancelPopup = false">否</el-button>
                <el-button @click="cancel" type="primary">是</el-button>
            </div>
        </PopUpFrame>

        <PopUpFrame  :isOpen.sync="isConfirmPopup">
            <div class="text--center mdTxt">已成功{{confirmTxt}}</div>
            <div class="text--center mt-30">
                <el-button @click="isConfirmPopup = false" type="primary">確定</el-button>
            </div>
        </PopUpFrame>

        <PopUpFrame :isOpen.sync="isRemoveAgentPopup">
            <div class="text--center mdTxt">是否移除顧問 <span class="text--primary">{{agentInfo.name}}</span>？</div>
            <div class="text--center mt-30">
                <el-button @click="isRemoveAgentPopup = false">否</el-button>
                <el-button @click="removeAgent" type="primary">是</el-button>
            </div>
        </PopUpFrame>
    </div>
</template>

<script lang="ts">
import { Vue, Component, Prop, Action, namespace } from 'nuxt-property-decorator';

import appointmentService from '~/shared/services/appointment.service';
import reviewsService from '~/shared/services/reviews.service';
import UtilsService from '~/shared/services/utils.service';
import { hideReviews } from '~/shared/const/hide-reviews';
import { Consultant, ConsultantWithAppointmentId } from '~/shared/models/consultant.model';
import { Appointment } from '~/shared/models/appointment.model';
import { SatisfactionType } from '~/shared/models/enum/satisfaction-type';
import { UserReviewParams } from '~/shared/models/reviews.model';

const localStorage = namespace('localStorage');
@Component({
    filters: {
        formatNumber(index: number) {
            if (index) {
                const upperNumber = ['零', '一', '二', '三', '四', '五', '六', '七', '八', '九', '十']
                return upperNumber[index];
            }
        },
        formatHopeContactTime(item: string): string {
        if (item) {
            const [hopeDay, hopeTime] = item.split('、');
            const day = hopeDay.split(',').length > 6 ? '不限日期' : hopeDay;
            const time = hopeTime.split(',').length > 3 ? '不限時間' : hopeTime;
            return `${day}、${time}`;
            }
            return '';
        }
    }
})
export default class ConsultantCard extends Vue {

    @Prop()
    agentInfo!: Consultant | ConsultantWithAppointmentId;

    @localStorage.State('current_role')
    currentRole!: string;

    isVisibleDialog = false;
    reviewsBtn = false;
    status = false;
    width: string = '';
    inputScore = 0;
    isCancelPopup = false;
    hideReviews = hideReviews;
    isConfirmPopup = false;
    isRemoveAgentPopup = false;
    confirmTxt = '';

    appointmentDetail: any = {
        age               : '',
        agentNo           : '',
        appointmentDate   : '',
        communicateStatus : '',
        consultantReadTime: null,
        consultantViewTime: null,
        contactTime       : '',
        contactType       : '',
        customerId        : 0,
        email             : '',
        gender            : '',
        hopeContactTime   : "",
        id                : 0,
        job               : "",
        lastModifiedDate  : '',
        name              : '',
        otherRequirement  : null,
        phone             : "",
        requirement       : '',
        appointmentScore : 0,
    };

    get notScoreAppointmentYet(): boolean {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      return isAppointment
            ? !this.agentInfo['appointmentScore']
            : !this.agentInfo.avgScore
    }

    get isAppointment(): boolean {
      return !!this.agentInfo['appointmentStatus'];
    }

    get latestNotClosedAppointment(): Appointment {
        return this.agentInfo.appointments!
                .filter((appointment) => appointment.communicateStatus === 'reserved' || appointment.communicateStatus === 'contacted')
                .map((reversedAppointment) => {
                    return {
                    ...reversedAppointment,
                    sortDate: new Date(reversedAppointment.appointmentDate)
                    }
                })
                .sort((preAppointment, nextAppointment) => +nextAppointment.sortDate - +preAppointment.sortDate)[0];
    }

    get showRemoveBtn(): boolean {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      if (isAppointment) {
        return false;
      } else {
        return this.agentInfo.contactStatus === 'picked' || !this.agentInfo.contactStatus;
      }
    }

    get avatarFileName() {
        return this.agentInfo.img;
    }

    get gender() {
        if (this.appointmentDetail.gender) {
            return this.appointmentDetail.gender === 'male' ? '男性' : '女性';
        }
        return ''
    }
    get actionBtnLabel() {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      if (isAppointment) {
        if (this.agentInfo['appointmentStatus'] === 'contacted') {
            return '已聯絡';
        }
        if (this.agentInfo['appointmentStatus'] === 'reserved') {
            return '已預約';
        }
        if (this.agentInfo['appointmentStatus'] === 'done') {
            return '已成交';
        }
        if (this.agentInfo['appointmentStatus'] === 'closed') {
            return '未成交';
        }
      } else {
        if (this.agentInfo.contactStatus === 'contacted') {
            return '已聯絡';
        }
        if (this.agentInfo.contactStatus === 'reserved') {
            return '已預約';
        }
      }
        return '進行預約';
    }

    get actionBtnStyle() {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      if (isAppointment) {
        if (this.agentInfo['appointmentStatus'] === 'contacted') {
            return 'contactedBtn';
        }
        if (this.agentInfo['appointmentStatus'] === 'reserved') {
            return 'reservedBtn';
        }
        if (this.agentInfo['appointmentStatus'] === 'done') {
            return 'doneBtn';
        }
        if (this.agentInfo['appointmentStatus'] === 'closed') {
            return 'closedBtn';
        }
      } else {
        if (this.agentInfo.contactStatus === 'contacted') {
            return 'contactedBtn';
        }
        if (this.agentInfo.contactStatus === 'reserved') {
            return 'reservedBtn';
        }
      }
    }

    get hopeContactTime() {
        const contactList = this.appointmentDetail.hopeContactTime
            .split("'").map((item: any) => item.slice(0, item.length));
        return contactList.filter((item: any) => !!item && item !== ",")
    }

    //////////////////////////////////////////////////////////////////////

    @Action
    removeFromMyConsultantList!: (agentNo: string) => Promise<boolean>;

    @Action
    storeConsultantList!: () => void;

    reservedOrShowAppointmentInfo() {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      const contactStatus = this.agentInfo.contactStatus;
        if (!isAppointment && (!contactStatus || contactStatus === 'picked')) {
            this.$router.push(`/questionnaire/${this.agentInfo.agentNo}`);
        } else {
            this.openPopUp();
        }
    }

    openPopUp(): void {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      const appointmentId = isAppointment
                          ? this.agentInfo['appointmentId']
                          : this.latestNotClosedAppointment.id;

        appointmentService.getAppointmentDetail(appointmentId!).then(res => {
            this.appointmentDetail = {
                ...res,
                appointmentScore:  this.agentInfo['appointmentScore'],
            };
            this.width = UtilsService.isMobileDevice() ? '80%' : '';
            this.isVisibleDialog = true;
        });
    }

    removeAgent() {
        this.removeFromMyConsultantList(this.agentInfo.agentNo).then((removeOk) => {
            this.isRemoveAgentPopup = false;
            setTimeout(() => {
                this.confirmTxt = '移除顧問';
                this.isConfirmPopup = true;
            }, 300);

        });
    }

    showAgentDetail(agentNo: string): void {
        this.$router.push(`/agentInfo/${agentNo}`);
    }

    userReviewsConsultants() {
      const isAppointment = !!this.agentInfo['appointmentStatus'];
      const appointmentId = isAppointment
                          ? this.agentInfo['appointmentId']
                          : this.latestNotClosedAppointment.id;

        const reviewParams: UserReviewParams = {
            appointmentId: appointmentId,
            score: this.inputScore,
            type : SatisfactionType.APPOINTMENT,
        }
        this.appointmentDetail.appointmentScore = this.inputScore;

        reviewsService.userReviewsConsultants(reviewParams).then((res) => {
            this.reviewsBtn = false;
            this.storeConsultantList();
        });
    }

    cancel() {
        appointmentService.cancelAppointment(this.latestNotClosedAppointment.id).then(res => {
            this.storeConsultantList();
            this.isVisibleDialog = false;
            this.isCancelPopup = false;
            setTimeout(() => {
                this.confirmTxt = '取消此筆預約';
                this.isConfirmPopup = true;

            }, 300);
        });
    }

    edit() {
        this.isVisibleDialog = false;
        this.$router.push({path: `/questionnaire/${this.agentInfo.agentNo}`, query: {'edit': 'true'}});
    }
}

</script>

<style lang="scss" scoped>
    .rowStyle {
        padding: 10px;
        background-color: $PRIMARY_WHITE;
        margin-bottom: 10px;
        display: flex;
        justify-content: space-between;
        border-left: 4px solid transparent;

        &.new {
            border-color: $YELLOW;
        }

        .satisfaction {
            font-size: 12px;
            font-weight: bold;
            margin-top: 5px;
        }
        .unfilled {
                font-size: 12px;
                padding-top: 5px;
                padding-left: 5px;
            }

        .professionals {
            height: 20px;
            overflow: hidden;
            white-space: nowrap;
            text-overflow: ellipsis;
            margin: 5px 0 10px 0;

            .professionalsTxt {
                font-size: 12px;
                font-weight: bold;
                margin-right: 5px;
            }
        }

        .delete {
            display: inline-block;
            color: $PRIMARY_RED;
            font-size: 14px;
            font-weight: bold;
            cursor: pointer;
        }

        .reservedBtn {
            color: $SKY_BLUE;
            border-color: $SKY_BLUE;

            &:focus {
                color: $PRIMARY_WHITE;
                background-color: $SKY_BLUE;
                opacity: 0.5;
            }
        }

        .contactedBtn {
            color: $GREEN;
            border-color: $GREEN;

            &:focus {
                color: $PRIMARY_WHITE;
                background-color: $GREEN;
                opacity: 0.5;
            }
        }

        .doneBtn {
            color: $BEIGE;
            border-color: $BEIGE;

            &:focus {
                color: $PRIMARY_WHITE;
                background-color: $BEIGE;
                opacity: 0.5;
            }
        }

        .closedBtn {
            color: $PRUDENTIAL_GREY;
            border-color: $PRUDENTIAL_GREY;

            &:focus {
                color: $PRIMARY_WHITE;
                background-color: $PRUDENTIAL_GREY;
                opacity: 0.5;
            }
        }

        .updateTime {
            font-size: 12px;
            font-weight: bold;
            color: #707070;
            text-align: right;
        }
    }

    .flex_column {
        display: flex;
        flex-direction: column;
        justify-content: space-between;
    }
</style>
